@@ -1,23 +1,19 @@
 import platform
 import re
 import pickle
-import urllib
 import requests
 from pathlib import Path
 import uuid
 from contextlib import contextmanager
-import os, sys
-import functools
-import inspect
-import warnings
 
+from sklearn.metrics import balanced_accuracy_score, accuracy_score
+import os
+import sys
 
 from mindsdb_native.__about__ import __version__
 from mindsdb_native.config import CONFIG
 from mindsdb_native.libs.data_types.mindsdb_logger import log
 from mindsdb_native.libs.constants.mindsdb import *
-import imagehash
-from PIL import Image
 
 
 def check_for_updates():
@@ -48,7 +44,7 @@ def check_for_updates():
     else:
         token = '{system}|{version}|{uid}'.format(system=platform.system(), version=__version__, uid=uuid_str)
         try:
-            open(mdb_file,'w').write(token)
+            open(mdb_file, 'w').write(token)
         except Exception:
             log.warning(f'Cannot store token, Please add write permissions to file: {mdb_file}')
             token = f'{token}.NO_WRITE'
@@ -58,19 +54,19 @@ def check_for_updates():
         ret = ret.json()
     except Exception as e:
         try:
-            log.warning(f'Got reponse: {ret} from update check server !')
+            log.warning(f'Got reponse: {ret} from update check server!')
         except Exception:
-            log.warning(f'Got no response from update check server !')
-        log.warning(f'Could not check for updates, got excetpion {e} !')
+            log.warning(f'Got no response from update check server!')
+        log.warning(f'Could not check for updates, got excetpion: {e}!')
         return
 
     try:
-        if 'version' in ret and ret['version']!= __version__:
-            log.warning("There is a new version of MindsDB {version}, please upgrade using:\npip3 uninstall mindsdb --upgrade".format(version=ret['version']))
+        if 'version' in ret and ret['version'] != __version__:
+            log.warning("There is a new version of MindsDB {version}, please upgrade using:\npip3 install mindsdb_native --upgrade".format(version=ret['version']))
         else:
             log.debug('MindsDB is up to date!')
     except Exception:
-        log.warning('could not check for MindsDB updates')
+        log.warning('Could not check for MindsDB updates')
 
 
 def convert_cammelcase_to_snake_string(cammel_string):
@@ -126,47 +122,59 @@ def get_value_bucket(value, buckets, col_stats, hmd=None):
     if buckets is None:
         return None
 
-    if col_stats['data_subtype'] in (DATA_SUBTYPES.SINGLE, DATA_SUBTYPES.MULTIPLE):
+    if col_stats['typing']['data_subtype'] in (DATA_SUBTYPES.SINGLE, DATA_SUBTYPES.MULTIPLE):
         if value in buckets:
             bucket = buckets.index(value)
         else:
             bucket = len(buckets) # for null values
 
-    elif col_stats['data_subtype'] in (DATA_SUBTYPES.BINARY, DATA_SUBTYPES.INT, DATA_SUBTYPES.FLOAT):
+    elif col_stats['typing']['data_subtype'] in (DATA_SUBTYPES.BINARY, DATA_SUBTYPES.INT, DATA_SUBTYPES.FLOAT):
         bucket = closest(buckets, value)
-    elif col_stats['data_subtype'] in (DATA_SUBTYPES.IMAGE) and hmd is not None:
-        bucket = hmd['bucketing_algorithms'][col_name].predict(np.array(imagehash.phash(Image.open(value)).reshape(1, -1)))[0]
     else:
         bucket = len(buckets) # for null values
 
     return bucket
 
 
-def evaluate_accuracy(predictions, full_dataset, col_stats, output_columns, hmd=None):
-    score = 0
-    for output_column in output_columns:
-        cummulative_scores = 0
-        if 'percentage_buckets' in col_stats[output_column]:
-            buckets = col_stats[output_column]['percentage_buckets']
+def evaluate_regression_accuracy(column, predictions, true_values):
+    pred_confidence_intervals = predictions[f'{column}_confidence_range']
+
+    within_interval = 0
+    for true, interval in zip(true_values, pred_confidence_intervals):
+        if true >= interval[0] and true <= interval[1]:
+            within_interval += 1
+    return within_interval/len(true_values)
+
+
+def evaluate_classification_accuracy(column, predictions, true_values):
+    pred_values = predictions[column]
+    return balanced_accuracy_score(true_values, pred_values)
+
+
+def evaluate_generic_accuracy(column, predictions, true_values):
+    pred_values = predictions[column]
+    return accuracy_score(true_values, pred_values)
+
+
+def evaluate_accuracy(predictions, data_frame, col_stats, output_columns, hmd=None):
+    column_scores = []
+    for column in output_columns:
+        col_type = col_stats[column]['typing']['data_type']
+        if col_type == DATA_TYPES.NUMERIC:
+            evaluator = evaluate_regression_accuracy
+        elif col_type == DATA_TYPES.CATEGORICAL:
+            evaluator = evaluate_classification_accuracy
         else:
-            buckets = None
+            evaluator = evaluate_generic_accuracy
+        column_score = evaluator(column, predictions, data_frame[column])
+        column_scores.append(column_score)
 
-        i = 0
-        for real_value in full_dataset[output_column]:
-            pred_val_bucket = get_value_bucket(predictions[output_column][i], buckets, col_stats[output_column], hmd)
-            if pred_val_bucket is None:
-                if predictions[output_column][i] == real_value:
-                    cummulative_scores += 1
-            elif pred_val_bucket == get_value_bucket(real_value, buckets, col_stats[output_column], hmd):
-                cummulative_scores += 1
-            i += 1
-
-        score += cummulative_scores/len(predictions[output_column])
-    score = score/len(output_columns)
+    score = sum(column_scores)/len(column_scores) if column_scores else 0.
     if score == 0:
         score = 0.00000001
     return score
 
+
 class suppress_stdout_stderr(object):
     def __init__(self):
         try:
@@ -236,77 +244,3 @@ def value_isnan(value):
     except:
         isnan = True
     return isnan
-
-
-def deprecated(reason):
-    """
-    This is a decorator which can be used to mark functions
-    as deprecated. It will result in a warning being emitted
-    when the function is used.
-    """
-    string_types = (type(b''), type(u''))
-
-    if isinstance(reason, string_types):
-
-        # The @deprecated is used with a 'reason'.
-        #
-        # .. code-block:: python
-        #
-        #    @deprecated("please, use another function")
-        #    def old_function(x, y):
-        #      pass
-
-        def decorator(func1):
-
-            if inspect.isclass(func1):
-                fmt1 = "Call to deprecated class {name} ({reason})."
-            else:
-                fmt1 = "Call to deprecated function {name} ({reason})."
-
-            @functools.wraps(func1)
-            def new_func1(*args, **kwargs):
-                warnings.simplefilter('always', DeprecationWarning)
-                warnings.warn(
-                    fmt1.format(name=func1.__name__, reason=reason),
-                    category=DeprecationWarning,
-                    stacklevel=2
-                )
-                warnings.simplefilter('default', DeprecationWarning)
-                return func1(*args, **kwargs)
-
-            return new_func1
-
-        return decorator
-
-    elif inspect.isclass(reason) or inspect.isfunction(reason):
-
-        # The @deprecated is used without any 'reason'.
-        #
-        # .. code-block:: python
-        #
-        #    @deprecated
-        #    def old_function(x, y):
-        #      pass
-
-        func2 = reason
-
-        if inspect.isclass(func2):
-            fmt2 = "Call to deprecated class {name}."
-        else:
-            fmt2 = "Call to deprecated function {name}."
-
-        @functools.wraps(func2)
-        def new_func2(*args, **kwargs):
-            warnings.simplefilter('always', DeprecationWarning)
-            warnings.warn(
-                fmt2.format(name=func2.__name__),
-                category=DeprecationWarning,
-                stacklevel=2
-            )
-            warnings.simplefilter('default', DeprecationWarning)
-            return func2(*args, **kwargs)
-
-        return new_func2
-
-    else:
-        raise TypeError(repr(type(reason)))
\ No newline at end of file
