@@ -1,3 +1,4 @@
+import string
 from collections import Counter, defaultdict
 
 import numpy as np
@@ -9,131 +10,27 @@ import imagehash
 from PIL import Image
 
 from mindsdb_native.libs.helpers.general_helpers import get_value_bucket
+from sklearn.neighbors import LocalOutlierFactor
 from mindsdb_native.libs.constants.mindsdb import *
 from mindsdb_native.libs.phases.base_module import BaseModule
-from mindsdb_native.libs.helpers.text_helpers import splitRecursive, clean_float
-from mindsdb_native.libs.phases.data_analyzer.scores import (
-    compute_duplicates_score,
-    compute_empty_cells_score,
-    compute_data_type_dist_score,
-    compute_similariy_score,
-    compute_value_distribution_score,
-    compute_z_score,
-    compute_lof_score,
-    compute_consistency_score,
-    compute_redundancy_score,
-    compute_variability_score,
-    compute_data_quality_score
+from mindsdb_native.libs.helpers.text_helpers import (
+    splitRecursive,
+    clean_float,
+    analyze_sentences,
+    get_language_dist
 )
-from mindsdb_native.libs.helpers.stats_helpers import sample_data
 
 
-def log_interesting_stats(log, stats):
-    """
-    # Provide interesting insights about the data to the user and send them to the logging server in order for it to generate charts
-    :param stats: The stats extracted up until this point for all columns
-    """
-    for col_name in stats:
-        col_stats = stats[col_name]
-        if 'is_empty' in col_stats and col_stats['is_empty'] == True:
-            continue
-        # Overall quality
-        if 'quality_score' in col_stats and col_stats['quality_score'] < 6:
-            # Some scores are not that useful on their own, so we should only warn users about them if overall quality is bad.
-            log.warning('Column "{}" is considered of low quality, the scores that influenced this decision will be listed below')
-            if 'duplicates_score' in col_stats and col_stats['duplicates_score'] < 6:
-                duplicates_percentage = col_stats['duplicates_percentage']
-                w = f'{duplicates_percentage}% of the values in column {col_name} seem to be repeated, this might indicate that your data is of poor quality.'
-                log.warning(w)
-                col_stats['duplicates_score_warning'] = w
-            else:
-                col_stats['duplicates_score_warning'] = None
-        else:
-            col_stats['duplicates_score_warning'] = None
+def lof_outliers(col_subtype, col_data):
+    lof = LocalOutlierFactor(contamination='auto')	
+    outlier_scores = lof.fit_predict(np.array(col_data).reshape(-1, 1)	)	
 
-        #Compound scores
-        if 'consistency_score' in col_stats and  col_stats['consistency_score'] < 3:
-            w = f'The values in column {col_name} rate poorly in terms of consistency. This means that the data has too many empty values, values with a hard to determine type and duplicate values. Please see the detailed logs below for more info'
-            log.warning(w)
-            col_stats['consistency_score_warning'] = w
-        else:
-            col_stats['consistency_score_warning'] = None
+    outliers = [col_data[i] for i in range(len(col_data)) if outlier_scores[i] < -0.8]	
 
-        if 'redundancy_score' in col_stats and  col_stats['redundancy_score'] < 5:
-            w = f'The data in the column {col_name} is likely somewhat redundant, any insight it can give us can already by deduced from your other columns. Please see the detailed logs below for more info'
-            log.warning(w)
-            col_stats['redundancy_score_warning'] = w
-        else:
-            col_stats['redundancy_score_warning'] = None
-
-        if 'variability_score' in col_stats and  col_stats['variability_score'] < 6:
-            w = f'The data in the column {col_name} seems to contain too much noise/randomness based on the value variability. That is to say, the data is too unevenly distributed and has too many outliers. Please see the detailed logs below for more info.'
-            log.warning(w)
-            col_stats['variability_score_warning'] = w
-        else:
-            col_stats['variability_score_warning'] = None
-
-        # Some scores are meaningful on their own, and the user should be warned if they fall below a certain threshold
-        if col_stats['empty_cells_score'] < 8:
-            empty_cells_percentage = col_stats['empty_percentage']
-            w = f'{empty_cells_percentage}% of the values in column {col_name} are empty, this might indicate that your data is of poor quality.'
-            log.warning(w)
-            col_stats['empty_cells_score_warning'] = w
-        else:
-            col_stats['empty_cells_score_warning'] = None
-
-        if col_stats['data_type_distribution_score'] < 7:
-            percentage_of_data_not_of_principal_type = col_stats['data_type_distribution_score'] * 100
-            principal_data_type = col_stats['data_type']
-            w = f'{percentage_of_data_not_of_principal_type}% of your data is not of type {principal_data_type}, which was detected to be the data type for column {col_name}, this might indicate that your data is of poor quality.'
-            log.warning(w)
-            col_stats['data_type_distribution_score_warning'] = w
-        else:
-            col_stats['data_type_distribution_score_warning'] = None
-
-        if 'z_test_based_outlier_score' in col_stats and col_stats['z_test_based_outlier_score'] < 6:
-            percentage_of_outliers = col_stats['z_test_based_outlier_score']*100
-            w = f"""Column {col_name} has a very high amount of outliers, {percentage_of_outliers}% of your data is more than 3 standard deviations away from the mean, this means that there might
-            be too much randomness in this column for us to make an accurate prediction based on it."""
-            log.warning(w)
-            col_stats['z_test_based_outlier_score_warning'] = w
-        else:
-            col_stats['z_test_based_outlier_score_warning'] = None
-
-        if 'lof_based_outlier_score' in col_stats and col_stats['lof_based_outlier_score'] < 4:
-            percentage_of_outliers = col_stats['percentage_of_log_based_outliers']
-            w = f"""Column {col_name} has a very high amount of outliers, {percentage_of_outliers}% of your data doesn't fit closely in any cluster using the KNN algorithm (20n) to cluster your data, this means that there might
-            be too much randomness in this column for us to make an accurate prediction based on it."""
-            log.warning(w)
-            col_stats['lof_based_outlier_score_warning'] = w
-        else:
-            col_stats['lof_based_outlier_score_warning'] = None
-
-        if 'value_distribution_score' in col_stats and col_stats['value_distribution_score'] < 3:
-            max_probability_key = col_stats['max_probability_key']
-            w = f"""Column {col_name} is very biased towards the value {max_probability_key}, please make sure that the data in this column is correct !"""
-            log.warning(w)
-            col_stats['value_distribution_score_warning'] = w
-        else:
-            col_stats['value_distribution_score_warning'] = None
-
-        if 'similarity_score' in col_stats and col_stats['similarity_score'] < 6:
-            similar_percentage = col_stats['max_similarity'] * 100
-            similar_col_name = col_stats['most_similar_column_name']
-            w = f'Column {col_name} and {similar_col_name} are {similar_percentage}% the same, please make sure these represent two distinct features of your data !'
-            log.warning(w)
-            col_stats['similarity_score_warning'] = w
-        else:
-            col_stats['similarity_score_warning'] = None
-
-        # We might want to inform the user about a few stats regarding his column regardless of the score, this is done below
-        log.info(f"""Data distribution for column "{col_name}" of type "{stats[col_name]['data_type']}" and subtype  "{stats[col_name]['data_subtype']}""")
-        try:
-            log.infoChart(stats[col_name]['data_subtype_dist'], type='list', uid='Data Type Distribution for column "{}"'.format(col_name))
-        except:
-            # Functionality is specific to mindsdb logger
-            pass
+    if col_subtype == DATA_SUBTYPES.INT:	
+        outliers = [int(x) for x in outliers]	
 
+    return outliers
 
 def clean_int_and_date_data(col_data, log):
     cleaned_data = []
@@ -224,7 +121,7 @@ def get_image_histogram(data):
 
 def get_histogram(data, data_type, data_subtype):
     """ Returns a histogram for the data and [optionaly] the percentage buckets"""
-    if data_subtype == DATA_SUBTYPES.TEXT:
+    if data_type == DATA_TYPES.TEXT:
         return get_text_histogram(data), None
     elif data_subtype == DATA_SUBTYPES.ARRAY:
         return get_hist(data), None
@@ -328,35 +225,28 @@ class DataAnalyzer(BaseModule):
     Additionally, also provides the user with some extra meaningful information about his data.
     """
 
-    # @TODO get rid of scores and stats entirely
-    def compute_scores(self, col_name, sample_df, full_data_dict, stats):
-        for score_func in [compute_duplicates_score,
-                           compute_empty_cells_score,
-                           compute_data_type_dist_score,
-                           compute_similariy_score,
-                           compute_value_distribution_score,
-                           ]:
-            score_out = score_func(stats, sample_df, col_name)
-            stats[col_name].update(score_out)
-
-        for score_func in [compute_z_score,
-                           compute_lof_score]:
-            score_out = score_func(stats, full_data_dict, col_name)
-            stats[col_name].update(score_out)
-
-        for score_func in [compute_consistency_score,
-                           compute_redundancy_score,
-                           compute_variability_score,
-                           compute_data_quality_score]:
-            score_out = score_func(stats, col_name)
-            stats[col_name].update(score_out)
-
     def run(self, input_data):
-        stats = self.transaction.lmd['column_stats']
         stats_v2 = self.transaction.lmd['stats_v2']
-        col_data_dict = {}
 
-        sample_df = input_data.sample_df
+        sample_settings = self.transaction.lmd['sample_settings']
+        if sample_settings['sample_for_analysis']:
+            sample_margin_of_error = sample_settings['sample_margin_of_error']
+            sample_confidence_level = sample_settings['sample_confidence_level']
+            sample_percentage = sample_settings['sample_percentage']
+            sample_function = self.transaction.hmd['sample_function']
+
+            sample_df = input_data.sample_df(sample_function,
+                                             sample_margin_of_error,
+                                             sample_confidence_level,
+                                             sample_percentage)
+
+            sample_size = len(sample_df)
+            population_size = len(input_data.data_frame)
+            self.transaction.log.info(f'Analyzing a sample of {sample_size} '
+                                      f'from a total population of {population_size}, '
+                                      f'this is equivalent to {round(sample_size * 100 / population_size, 1)}% of your data.')
+        else:
+            sample_df = input_data.data_frame
 
         for col_name in self.transaction.lmd['empty_columns']:
             stats_v2[col_name] = {}
@@ -371,13 +261,9 @@ class DataAnalyzer(BaseModule):
             col_data = sample_df[col_name].dropna()
             if data_type == DATA_TYPES.NUMERIC or data_subtype == DATA_SUBTYPES.TIMESTAMP:
                 col_data = clean_int_and_date_data(col_data, self.log)
-            col_data_dict[col_name] = col_data
 
             stats_v2[col_name]['empty'] = get_column_empty_values_report(input_data.data_frame[col_name])
 
-            stats[col_name]['empty_cells'] = stats_v2[col_name]['empty']['empty_cells']
-            stats[col_name]['empty_percentage'] = stats_v2[col_name]['empty']['empty_percentage']
-
             if data_type == DATA_TYPES.CATEGORICAL:
                 hist_data = input_data.data_frame[col_name]
                 stats_v2[col_name]['unique'] = get_uniq_values_report(input_data.data_frame[col_name])
@@ -389,8 +275,6 @@ class DataAnalyzer(BaseModule):
                                                           data_subtype=data_subtype)
             stats_v2[col_name]['histogram'] = histogram
             stats_v2[col_name]['percentage_buckets'] = percentage_buckets
-            stats[col_name]['histogram'] = histogram
-            stats[col_name]['percentage_buckets'] = percentage_buckets
             if histogram:
                 S, biased_buckets = compute_entropy_biased_buckets(histogram['y'], histogram['x'])
                 stats_v2[col_name]['bias'] = {
@@ -406,19 +290,26 @@ class DataAnalyzer(BaseModule):
                         warning_str = "You may want to check if you see something suspicious on the right-hand-side graph."
                     stats_v2[col_name]['bias']['warning'] = warning_str + " This doesn't necessarily mean there's an issue with your data, it just indicates a higher than usual probability there might be some issue."
 
-            self.compute_scores(col_name, sample_df, col_data_dict, stats)
+                if data_type == DATA_TYPES.NUMERIC:
+                        outliers = lof_outliers(data_subtype, col_data)
+                        stats_v2[col_name]['outliers'] = {	
+                            'outlier_values': outliers,
+                            'outlier_buckets': compute_outlier_buckets(outlier_values=outliers,	
+                                                                    hist_x=histogram['x'],	
+                                                                    hist_y=histogram['y'],	
+                                                                    percentage_buckets=percentage_buckets,	
+                                                                    col_stats=stats_v2[col_name]),	
+                            'description': """Potential outliers can be thought as the "extremes", i.e., data points that are far from the center of mass (mean/median/interquartile range) of the data."""	
+                        }
 
-            if 'lof_outliers' in stats[col_name]:
-                stats_v2[col_name]['outliers'] = {
-                    'outlier_values': stats[col_name]['lof_outliers'],
-                    'outlier_score': stats[col_name]['lof_based_outlier_score'],
-                    'outlier_buckets': compute_outlier_buckets(outlier_values=stats[col_name]['lof_outliers'],
-                                                               hist_x=histogram['x'],
-                                                               hist_y=histogram['y'],
-                                                               percentage_buckets=percentage_buckets,
-                                                               col_stats=stats[col_name]),
-                    'description': """Potential outliers can be thought as the "extremes", i.e., data points that are far from the center of mass (mean/median/interquartile range) of the data."""
-                }
+            if data_type == DATA_TYPES.TEXT:
+                lang_dist = get_language_dist(col_data)
+                nr_words, word_dist, nr_words_dist = analyze_sentences(col_data)
+
+                stats_v2[col_name]['avg_words_per_sentence'] = nr_words / len(col_data)
+                stats_v2[col_name]['word_dist'] = dict(word_dist)
+                stats_v2[col_name]['nr_words_dist'] = dict(nr_words_dist)
+                stats_v2[col_name]['lang_dist'] = lang_dist
 
             stats_v2[col_name]['nr_warnings'] = 0
             for x in stats_v2[col_name].values():
@@ -427,9 +318,7 @@ class DataAnalyzer(BaseModule):
                 stats_v2[col_name]['nr_warnings'] += 1
             self.log.info(f'Finished analyzing column: {col_name} !\n')
 
-        log_interesting_stats(self.log, stats)
-
-        self.transaction.lmd['data_preparation']['accepted_margin_of_error'] = self.transaction.lmd['sample_margin_of_error']
+        self.transaction.lmd['data_preparation']['accepted_margin_of_error'] = self.transaction.lmd['sample_settings']['sample_margin_of_error']
 
         self.transaction.lmd['data_preparation']['total_row_count'] = len(input_data.data_frame)
         self.transaction.lmd['data_preparation']['used_row_count'] = len(sample_df)
